"use strict";
const chai = require("chai");
const chaiAsPromised = require("chai-as-promised");
const knex = require("knex");
const typed_sql_1 = require("typed-sql");
const _1 = require("../");
chai.use(chaiAsPromised);
let expect = chai.expect;
describe("SQL Graph", function () {
    let mappings = {
        orderDetail: typed_sql_1.defineMapping('order_detail', {
            id: typed_sql_1.defineNumber(),
            orderId: typed_sql_1.defineNumber(),
            productId: typed_sql_1.defineNumber(),
            quantity: typed_sql_1.defineNumber(),
            unitPrice: typed_sql_1.defineNumber()
        }),
        order: typed_sql_1.defineMapping('order', {
            id: typed_sql_1.defineNumber()
        }),
        product: typed_sql_1.defineMapping('product', {
            id: typed_sql_1.defineNumber(),
            categoryId: typed_sql_1.defineNumber()
        }),
        category: typed_sql_1.defineMapping('category', {
            id: typed_sql_1.defineNumber(),
            name: typed_sql_1.defineString()
        })
    };
    let knexClient = knex({
        client: 'pg'
    });
    let mapper = new typed_sql_1.Mapper(knexClient);
    let sqlGraph = new _1.SqlGraph();
    sqlGraph.addMasterDetailJoin(mappings.orderDetail, mappings.order, typed_sql_1.equal(mappings.orderDetail.orderId, mappings.order.id));
    sqlGraph.addMasterDetailJoin(mappings.orderDetail, mappings.product, typed_sql_1.equal(mappings.orderDetail.productId, mappings.product.id));
    sqlGraph.addMasterDetailJoin(mappings.product, mappings.category, typed_sql_1.equal(mappings.product.categoryId, mappings.category.id));
    it("should compute simple join paths", function () {
        let joinPath = sqlGraph.getJoinPath(mappings.orderDetail, mappings.category);
        expect(joinPath).to.not.be.null;
        expect(joinPath.length).to.equal(2);
    });
    it("should error on non-existing join paths", function () {
        expect(function () { sqlGraph.getJoinPath(mappings.category, mappings.order); }).to.throw(_1.SqlGraphTopologyError);
    });
    it("should generate empty join paths for identity mappings", function () {
        let joinPath = sqlGraph.getJoinPath(mappings.orderDetail, mappings.orderDetail);
        expect(joinPath.length).to.equal(0);
    });
    it("should generate from queries with joins corresponding to the join path", function () {
        let joinPath = sqlGraph.getJoinPath(mappings.orderDetail, mappings.category);
        let query = _1.createJoinQuery(mapper, joinPath)
            .select({
            orderDetailId: mappings.orderDetail.id,
            categoryId: mappings.category.id
        });
        expect(query.getKnexQuery().toQuery()).to.equal('select "order_detail"."id" as "orderDetailId", "category"."id" as "categoryId" from "order_detail" inner join "product" on "order_detail"."productId" = "product"."id" inner join "category" on "product"."categoryId" = "category"."id"');
    });
});
//# sourceMappingURL=core.test.js.map